@@ -4,8 +4,8 @@
 
 ;; Author: SAKURAI Masashi <m.sakurai at kiwanami.net>
 ;; Maintainer: Al Haji-Ali <abdo.haji.ali at gmail.com>
-;; Package-Version: 20251018.1103
-;; Package-Revision: 98279aa29c15
+;; Package-Version: 20251023.2205
+;; Package-Revision: de895bc8071b
 ;; Keywords: calendar, org
 ;; Package-Requires: ((emacs "28.1") (calfw "2.0"))
 ;; URL: https://github.com/haji-ali/emacs-calfw
@@ -34,7 +34,7 @@
 ;; ;; use org agenda buffer style keybinding.
 ;; ;; (setq calfw-org-overwrite-default-keybinding t)
 ;;
-;; M-x calfw-open-org-calendar
+;; M-x calfw-org-open-calendar
 
 ;;; Code:
 
@@ -70,11 +70,6 @@ For example:
   "Default arguments for collecting agenda entries.
 If value is nil, `org-agenda-entry-types' is used.")
 
-(defvar calfw-org-icalendars nil
-  "Org buffers for exporting icalendars.
-Setting a list of the custom agenda files, one can use the
-different agenda files from the default agenda ones.")
-
 (defvar calfw-org-overwrite-default-keybinding nil
   "Overwrite default keybinding.
 
@@ -97,27 +92,51 @@ v m | `calfw-change-view-month'
 (defvar calfw-org-face-agenda-item-foreground-color "Seagreen4"
   "Variable for org agenda item foreground color.")
 
-(defun calfw-org--collect-schedules-period (begin end)
-  "Return org schedule items between BEGIN and END."
+(defun calfw-org--collect-schedules-period (org-files begin end)
+  "Return org schedule items between BEGIN and END from ORG-FILES."
   (let ((org-agenda-prefix-format " "))
-    (setq org-agenda-buffer
-          (when (buffer-live-p org-agenda-buffer)
-            org-agenda-buffer))
+    ;; @AL: This seems wrong
+    ;; (setq org-agenda-buffer
+    ;;       (when (buffer-live-p org-agenda-buffer)
+    ;;         org-agenda-buffer))
     (org-compile-prefix-format nil)
-    (cl-loop for date in (calfw-enumerate-days begin end) append
-             (cl-loop for file in (or calfw-org-icalendars (org-agenda-files nil 'ifmode))
-                      append
-                      (progn
-                        (org-check-agenda-file file)
-                        (apply 'org-agenda-get-day-entries
-                               file date
-                               calfw-org-agenda-schedule-args))))))
+
+    ;; We need several hacks to resolve issues like those in #91.
+    ;;
+    ;; Org diary for schedules/deadline is weird. It returns the first date an
+    ;; item is scheduled, then does not return any that were scheduled before
+    ;; today but after that first date (and no real way to control "today").
+    ;; To resolve, we employ an incomplete "feature" using
+    ;; `org-extend-today-until' to redefine that "today" is one second before
+    ;; the start of the current date.
+
+    ;; Then it returns all future events if `org-agenda-show-future-repeats'
+    ;; is non-nil, but the `date' property is set to the date of the first
+    ;; scheduled date if it is in the past relative to today. To resolve this,
+    ;; we instead update the value of `date' to be our input.
+    (let ((now (float-time (current-time))))
+      (cl-loop
+       for date in (calfw-enumerate-days begin end)
+       for target = (float-time (calfw-calendar-to-emacs date))
+       for org-extend-today-until = (max 0 (/ (- now target -1) 3600))
+       append
+       (cl-loop
+        for file in org-files
+        append
+        (progn
+          (org-check-agenda-file file)
+          ;; We set the date here, since we cannot rely on the
+          ;; existing `date' property for schedules and
+          ;; deadlines.
+          (mapcar (lambda (entry) (calfw--tp entry 'date date))
+                  (apply 'org-agenda-get-day-entries
+                         file date
+                         calfw-org-agenda-schedule-args))))))))
 
 (defun calfw-org-onclick ()
   "Jump to the clicked org item."
   (interactive)
-  (let (
-        (marker (get-text-property (point) 'org-marker))
+  (let ((marker (get-text-property (point) 'org-marker))
         (link   (get-text-property (point) 'org-link))
         (file   (get-text-property (point) 'cfw:org-file))
         (beg    (get-text-property (point) 'cfw:org-h-beg))
@@ -194,7 +213,9 @@ ITEM is an org entry.  Return a string with text properties."
          (extra (calfw-org--tp item 'extra)))
     (setq text (substring-no-properties text))
     (when (and extra (string-match (concat "^" org-deadline-string ".*") extra))
-      (add-text-properties 0 (length text) (list 'face (org-agenda-deadline-face 1.0)) text))
+      (add-text-properties 0 (length text) (list 'face
+                                                 (org-agenda-deadline-face 1.0))
+                           text))
     (if org-todo-keywords-for-agenda
         (when (string-match (concat "^[\t ]*\\<\\(" (mapconcat 'identity org-todo-keywords-for-agenda "\\|") "\\)\\>") text)
           (add-text-properties (match-beginning 1) (match-end 1) (list 'face (org-get-todo-face (match-string 1 text))) text)))
@@ -240,47 +261,57 @@ those string in multi-lines.")
     (calendar-gregorian-from-absolute date))
    (t date)))
 
-(defun calfw-org-get-timerange (text)
+(defun calfw-org-get-timerange (text &optional at-begin)
   "Return a range object (begin end text).
-If TEXT does not have a range, return nil."
-  (let* ((dotime (calfw-org--tp text 'dotime)))
-    (and (stringp dotime) (and dotime (string-match org-ts-regexp dotime))
-	 (let ((date-string  (match-string 1 dotime))
-	       (extra (calfw-org--tp text 'extra)))
-	   (if (and extra (string-match "(\\([0-9]+\\)/\\([0-9]+\\)): " extra))
-	       (let* ((cur-day (string-to-number
-				(match-string 1 extra)))
-		      (total-days (string-to-number
-				   (match-string 2 extra)))
-		      (start-date (time-subtract
-				   (org-read-date nil t date-string)
-				   (seconds-to-time (* 3600 24 (- cur-day 1)))))
-		      (end-date (time-add
-				 (org-read-date nil t date-string)
-				 (seconds-to-time (* 3600 24 (- total-days cur-day))))))
-		 (list (calendar-gregorian-from-absolute (time-to-days start-date))
-		       (calendar-gregorian-from-absolute
-                        (time-to-days end-date))
-                       text)))))))
+If TEXT does not have a range, return nil.
+If AT-BEGIN is non-nil, then it should be the date of the first
+day in the calendar."
+  (let* ((dotime (calfw-org--tp text 'dotime))
+         date-string)
+    (and (or (and (stringp dotime)
+                  (string-match org-ts-regexp dotime)
+                  (setq date-string (match-string 1 dotime)))
+             at-begin)
+	 (let ((extra (calfw-org--tp text 'extra)))
+	   (when (and extra (string-match "(\\([0-9]+\\)/\\([0-9]+\\)): " extra))
+	     (let* ((cur-day (string-to-number
+			      (match-string 1 extra)))
+		    (total-days (string-to-number
+				 (match-string 2 extra)))
+                    (date (if date-string
+                              (org-read-date nil t date-string)
+                            (calfw-calendar-to-emacs at-begin)))
+		    (start-date (time-subtract
+				 date
+                                 ;; Add an extra second to ensure robustness
+				 (seconds-to-time (1- (* 3600 24 (- cur-day 1))))))
+		    (end-date (time-add
+			       date
+			       (seconds-to-time (* 3600 24 (- total-days cur-day))))))
+	       (list (calendar-gregorian-from-absolute (time-to-days start-date))
+		     (calendar-gregorian-from-absolute
+                      (time-to-days end-date))
+                     text)))))))
 
-(defun calfw-org--schedule-period-to-calendar (begin end)
-  "Return calfw calendar items between BEGIN and END from org schedule data."
+(defun calfw-org--schedule-period-to-calendar (org-files begin end)
+  "Return calfw calendar items between BEGIN and END from ORG-FILES."
   (cl-loop
    ;;with calfw-org-todo-keywords-regexp = (regexp-opt org-todo-keywords-for-agenda) ; dynamic bind
    with contents = nil with periods = nil
-   for i in (calfw-org--collect-schedules-period begin end)
+   for i in (calfw-org--collect-schedules-period org-files begin end)
    for date = (calfw-org--tp i 'date)
    for line = (funcall calfw-org-schedule-summary-transformer i)
-   for range = (calfw-org-get-timerange line)
-   if range do
+   for range = (calfw-org-get-timerange line (and (equal date begin) date))
+   if range
+   do
    (unless (member range periods)
      (push range periods))
    else do
-                                        ; dotime is not present if this event was already added as a timerange
-   (if (calfw-org--tp i 'dotime)
-       (setq contents (calfw--contents-add
-		       (calfw-org-normalize-date date)
-		       line contents)))
+   ;; dotime is not present if this event was already added as a timerange
+   (when (calfw-org--tp i 'dotime)
+     (setq contents (calfw--contents-add
+		     (calfw-org-normalize-date date)
+		     line contents)))
    finally return (nconc contents (list (cons 'periods periods)))))
 
 (defun calfw-org--schedule-sorter (text1 text2)
@@ -489,22 +520,28 @@ Open the `org-agenda' buffer for the date at point, DATE."
      ("SPC" . calfw-org-open-agenda-day)))
   "Key map for the calendar buffer.")
 
-(defun calfw-org-create-source (&optional color)
-  "Create an `org-agenda' source.
-
-COLOR, if given, is the color to use.
-Returns a new calfw source."
+(defun calfw-org-create-source (name org-files color)
+  "Create a calfw source named NAME with org files ORG-FILES and COLOR."
   (make-calfw-source
-   :name "org-agenda"
-   :color (or color calfw-org-face-agenda-item-foreground-color)
-   :data 'calfw-org--schedule-period-to-calendar))
+   :name name :color color
+   :data (apply-partially 'calfw-org--schedule-period-to-calendar org-files)))
 
-(defun calfw-org-open-calendar ()
-  "Open an org schedule calendar in the new buffer."
+(cl-defun calfw-org-open-calendar (&optional name org-files)
+  "Open an org schedule calendar in a new buffer.
+
+Events are taken from ORG-FILES (defaults to those returned by
+`org-agenda-files') and NAME is the name of the calendar
+(defaults to \"org-agenda\")."
   (interactive)
   (save-excursion
-    (let* ((source1 (calfw-org-create-source))
-           (curr-keymap (if calfw-org-overwrite-default-keybinding calfw-org-custom-map calfw-org-schedule-map))
+    (let* ((source1 (calfw-org-create-source
+                     (or name "org-agenda")
+                     (or org-files
+                         (org-agenda-files nil 'ifmode))
+                     calfw-org-face-agenda-item-foreground-color))
+           (curr-keymap (if calfw-org-overwrite-default-keybinding
+                            calfw-org-custom-map
+                          calfw-org-schedule-map))
            (cp (calfw-create-calendar-component-buffer
                 :view 'month
                 :contents-sources (list source1)
